import React, { Component } from 'react';
import pathToRegexp from 'path-to-regexp';
import moment from 'moment';
import { connect } from 'dva';
import { routerRedux } from 'dva/router';
import { Card, Modal, Button, message } from 'antd';
import { StandardTableList } from '../../../components/RBList/index';
import { renderStatus, addRowKey } from '../../../utils/utils';
import styles from './ShelvesList.less';

const Message = message;

function firstLetterUpper(str) {
  return str.substring(0, 1).toUpperCase() + str.substring(1);
}
function getItemName(str) {
  if (str === 'course') {
    return '课程';
  } else if (str === 'support') {
    return '配套';
  } else if (str === 'package') {
    return '套餐包';
  } else {
    return '产品';
  }
}

@connect(({ loading, shelves }) => ({
  shelves,
  loading: loading.models.shelves,
}))
export default class ShelvesListPage extends Component {
  constructor(props) {
    super(props);
    const { state, pathname } = props.location;
    const match = pathToRegexp('/shelves/:itemType').exec(pathname);
    this.state = {
      UIParams: (state || {}).UIParams, // 组件的状态参数
      Queryers: (state || {}).Queryers, // 查询的条件参数
      itemType: match[1], // 货架产品类型
    };
  }
  componentDidMount() {
    const { itemType } = this.state;
    this.props.dispatch({
      type: `shelves/fetch${firstLetterUpper(itemType)}ItemList`,
      payload: { ...this.state.Queryers },
    });
  }
  handleCreateOperation = () => {
    this.props.dispatch(routerRedux.push({
      pathname: '/shelves/create',
      state: this.state,
    }));
  }
  handleDeleteOperation = (item) => {
    const { itemType } = this.state;
    Modal.confirm({
      okText: '确定',
      cancelText: '取消',
      title: `你确定要下架该${getItemName(itemType)}吗?`,
      onOk: () => {
        this.props.dispatch({
          type: 'shelves/deleteItem',
          payload: { id: item.id },
          states: this.state,
        });
      },
    });
  }
  handleEditOperation = (item) => {
    const { itemType } = this.state;
    this.props.dispatch(routerRedux.push({
      pathname: '/shelves/edit',
      state: {
        pid: item.pid,
        merchantId: item.merchantId,
        productType: itemType,
        ...this.state,
      },
    }));
  }
  handleFilterOperation = (params, states) => {
    const { itemType }  = this.state;
    this.props.dispatch({
      type: `shelves/fetch${firstLetterUpper(itemType)}ItemList`,
      payload: params,
    });
    this.setState({
      UIParams: states,
      Queryers: params,
    });
  }
  handleBatchOperation = () => {
    Message.info('暂不支持批量操作!');
  }

  render() {
    const { itemType } = this.state;
    const { loading, shelves } = this.props;
    const { list, totalSize, pageSize, pageNo } = shelves;

    const renderOperation = (item) => {
      return (
        <div>
          <Button
            size="small"
            className={styles.editBtn}
            onClick={() => this.handleEditOperation(item)}
          >编辑
          </Button>
          <Button
            size="small"
            className={styles.delBtn}
            onClick={() => this.handleDeleteOperation(item)}
          >下架
          </Button>
        </div>
      );
    };

    const batchActions = [{
      key: 'delete',
      name: '批量删除',
    }, {
      key: 'recovery',
      name: '批量恢复',
    }];
    const basicSearch = {
      keys: [{
        name: `${getItemName(itemType)}编号`,
        field: 'code',
      }, {
        name: `${getItemName(itemType)}名称`,
        field: 'name',
      }],
    };
    const pagination = {
      pageNo,
      pageSize,
      totalSize,
    };
    const columns = [{
      title: `${getItemName(itemType)}编号`,
      key: 1,
      dataIndex: 'code',
      render: (text, record) => (
        <a
          className={styles.link}
          onClick={() => this.handleEditOperation(record)}
        >
          {text}
        </a>
      ),
      width: '15%',
    }, {
      title: `${getItemName(itemType)}名称`,
      key: 2,
      dataIndex: 'name',
      render: (text, record) => (
        <a
          className={styles.link}
          onClick={() => this.handleEditOperation(record)}
        >
          {text}
        </a>
      ),
      width: '30%',
    }, {
      title: '上架渠道',
      key: 3,
      dataIndex: 'merchantName',
      width: '14%',
    }, {
      title: '上架状态',
      key: 4,
      dataIndex: 'status',
      render: text => renderStatus(text),
      width: '10%',
    }, {
      title: '更新时间',
      key: 5,
      dataIndex: 'gmtModified',
      render: text => moment(text).format('YYYY-MM-DD HH:mm:ss'),
      width: '17%',
    }, {
      title: '操作',
      key: 6,
      dataIndex: 'operation',
      render: (_, record) => renderOperation(record),
      width: '13%',
    }];
    return (
      <Card>
        <StandardTableList
          columns={columns}
          loading={loading}
          dataSource={addRowKey(list)}
          header={{
            basicSearch,
            onFilterClick: this.handleFilterOperation,
            onCreateClick: this.handleCreateOperation,
          }}
          footer={{
            pagination,
            batchActions,
            onBatchClick: this.handleBatchOperation,
          }}
          keepUIState={{ ...this.state.UIParams }}
        />
      </Card>
    );
  }
}
